"""
BORIS
Behavioral Observation Research Interactive Software
Copyright 2012-2024 Olivier Friard

This file is part of BORIS.

  BORIS is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 3 of the License, or
  any later version.

  BORIS is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not see <http://www.gnu.org/licenses/>.

"""

import logging
import json

from PyQt5.QtGui import QIcon
from PyQt5.QtWidgets import QDialog, QWidget, QFileDialog, QMessageBox

from . import dialog
from .add_modifier_ui import Ui_Dialog
from . import config as cfg
from .utilities import sorted_keys


class addModifierDialog(QDialog, Ui_Dialog):
    tabMem = -1
    itemPositionMem = -1

    def __init__(self, modifiers_str, subjects=[], parent=None):
        super().__init__()
        self.setupUi(self)

        self.subjects = subjects
        if not self.subjects:
            self.pb_add_subjects.setEnabled(False)

        self.pbAddModifier.clicked.connect(self.addModifier)
        self.pbAddModifier.setIcon(QIcon(":/frame_forward"))
        self.pbAddSet.clicked.connect(self.addSet)
        self.pbRemoveSet.clicked.connect(self.removeSet)
        self.pbModifyModifier.clicked.connect(self.modifyModifier)
        self.pbModifyModifier.setIcon(QIcon(":/frame_backward"))

        self.pbMoveUp.clicked.connect(self.moveModifierUp)
        self.pbMoveDown.clicked.connect(self.moveModifierDown)
        self.pbMoveSetLeft.clicked.connect(self.moveSetLeft)
        self.pbMoveSetRight.clicked.connect(self.moveSetRight)
        self.pbRemoveModifier.clicked.connect(self.removeModifier)
        self.pb_sort_modifiers.clicked.connect(self.sort_modifiers)
        self.pb_add_subjects.clicked.connect(self.add_subjects)
        self.pb_load_file.clicked.connect(self.add_modifiers_from_file)

        self.pbOK.clicked.connect(lambda: self.pb_pushed("ok"))
        self.pbCancel.clicked.connect(lambda: self.pb_pushed("cancel"))

        self.le_name.textChanged.connect(self.set_name_changed)
        self.le_description.textChanged.connect(self.set_description_changed)

        self.cbType.currentIndexChanged.connect(self.type_changed)

        dummy_dict = json.loads(modifiers_str) if modifiers_str else {}
        modif_values = []
        for idx in sorted_keys(dummy_dict):
            modif_values.append(dummy_dict[idx])

        self.modifiers_sets_dict = {}
        for modif in modif_values:
            self.modifiers_sets_dict[str(len(self.modifiers_sets_dict))] = dict(modif)

        self.tabWidgetModifiersSets.currentChanged.connect(self.tabWidgetModifiersSets_changed)

        # create tab
        for idx in sorted_keys(self.modifiers_sets_dict):
            self.tabWidgetModifiersSets.addTab(QWidget(), f"Set #{int(idx) + 1}")

        if self.tabWidgetModifiersSets.currentIndex() == -1:
            for w in (
                self.lb_name,
                self.le_name,
                self.lbType,
                self.lbValues,
                self.lb_description,
                self.le_description,
                self.cbType,
                self.lwModifiers,
                self.pbMoveUp,
                self.pbMoveDown,
                self.pbRemoveModifier,
                self.pbRemoveSet,
                self.pbMoveSetLeft,
                self.pbMoveSetRight,
                self.pb_add_subjects,
                self.pb_load_file,
                self.pb_sort_modifiers,
            ):
                w.setVisible(False)
            for w in (self.leModifier, self.leCode, self.pbAddModifier, self.pbModifyModifier):
                w.setEnabled(False)

        # set first tab as active
        self.tabMem = 0

    def pb_pushed(self, button):
        if self.leModifier.text():
            if (
                dialog.MessageDialog(
                    cfg.programName,
                    (
                        "You are working on a behavior.<br>"
                        "If you close the window it will be lost.<br>"
                        "Do you want to change modifiers set"
                    ),
                    ["Close", cfg.CANCEL],
                )
                == cfg.CANCEL
            ):
                return
        if button == "ok":
            self.accept()
        if button == "cancel":
            self.reject()

    def add_subjects(self):
        """
        add subjects as modifiers
        """

        for subject, key in self.subjects:
            if self.itemPositionMem != -1:
                self.lwModifiers.insertItem(self.itemPositionMem, f"{subject} ({key})" if key else subject)
            else:
                self.lwModifiers.addItem(f"{subject} ({key})" if key else subject)

        self.modifiers_sets_dict[str(self.tabWidgetModifiersSets.currentIndex())]["values"] = [
            self.lwModifiers.item(x).text() for x in range(self.lwModifiers.count())
        ]

    def add_modifiers_from_file(self):
        """
        add modifiers from file
        """

        fn = QFileDialog().getOpenFileName(self, "Load modifiers from file", "", "All files (*)")
        file_name = fn[0] if type(fn) is tuple else fn
        if file_name:
            try:
                with open(file_name) as f_in:
                    for line in f_in:
                        if line.strip():
                            for c in cfg.CHAR_FORBIDDEN_IN_MODIFIERS:
                                if c in line.strip():
                                    QMessageBox.critical(
                                        self,
                                        cfg.programName,
                                        (
                                            f"The character <b>{c}</b> is not allowed.<br>"
                                            "The following characters are not allowed in modifiers:<br>"
                                            f"<b>{cfg.CHAR_FORBIDDEN_IN_MODIFIERS}</b>"
                                        ),
                                    )
                                    break
                            else:
                                if line.strip() not in [self.lwModifiers.item(x).text() for x in range(self.lwModifiers.count())]:
                                    if self.itemPositionMem != -1:
                                        self.lwModifiers.insertItem(self.itemPositionMem, line.strip())
                                    else:
                                        self.lwModifiers.addItem(line.strip())

                self.modifiers_sets_dict[str(self.tabWidgetModifiersSets.currentIndex())]["values"] = [
                    self.lwModifiers.item(x).text() for x in range(self.lwModifiers.count())
                ]
            except Exception:
                QMessageBox.warning(self, cfg.programName, f"Error reading modifiers from file:<br>{file_name}")
                logging.warning(f"Error reading modifiers from file<br>{file_name}")

    def sort_modifiers(self):
        """
        sort modifiers
        """

        modifiers = sorted([self.lwModifiers.item(x).text() for x in range(self.lwModifiers.count())])
        self.lwModifiers.clear()
        for modifier in modifiers:
            if self.itemPositionMem != -1:
                self.lwModifiers.insertItem(self.itemPositionMem, modifier)
            else:
                self.lwModifiers.addItem(modifier)

        self.modifiers_sets_dict[str(self.tabWidgetModifiersSets.currentIndex())]["values"] = [
            self.lwModifiers.item(x).text() for x in range(self.lwModifiers.count())
        ]

    def set_name_changed(self):
        """
        set name changed
        """
        if not self.modifiers_sets_dict:
            self.modifiers_sets_dict["0"] = {"name": "", "description": "", "type": cfg.SINGLE_SELECTION, "values": []}
        self.modifiers_sets_dict[str(self.tabWidgetModifiersSets.currentIndex())]["name"] = self.le_name.text().strip()

    def set_description_changed(self):
        """
        set description changed
        """
        if not self.modifiers_sets_dict:
            self.modifiers_sets_dict["0"] = {"name": "", "description": "", "type": cfg.SINGLE_SELECTION, "values": []}
        self.modifiers_sets_dict[str(self.tabWidgetModifiersSets.currentIndex())]["description"] = self.le_description.text().strip()

    def type_changed(self):
        """
        type changed
        """
        if not self.modifiers_sets_dict:
            self.modifiers_sets_dict["0"] = {"name": "", "description": "", "type": cfg.SINGLE_SELECTION, "values": []}
        self.modifiers_sets_dict[str(self.tabWidgetModifiersSets.currentIndex())]["type"] = self.cbType.currentIndex()
        # disable if modifier numeric or value from external data file
        for obj in (
            self.lbValues,
            self.lwModifiers,
            self.leModifier,
            self.leCode,
            self.lbModifier,
            self.lbCode,
            self.lbCodeHelp,
            self.pbMoveUp,
            self.pbMoveDown,
            self.pbRemoveModifier,
            self.pb_add_subjects,
            self.pbAddModifier,
            self.pbModifyModifier,
            self.pb_load_file,
            self.pb_sort_modifiers,
        ):
            obj.setEnabled(self.cbType.currentIndex() not in [cfg.NUMERIC_MODIFIER, cfg.EXTERNAL_DATA_MODIFIER])
        if self.cbType.currentIndex() == cfg.EXTERNAL_DATA_MODIFIER:
            self.lb_name.setText("Variable name")
        else:
            self.lb_name.setText("Set name")

    def moveSetLeft(self):
        """
        move selected modifiers set left
        """
        if self.tabWidgetModifiersSets.currentIndex():
            (
                self.modifiers_sets_dict[str(self.tabWidgetModifiersSets.currentIndex() - 1)],
                self.modifiers_sets_dict[str(self.tabWidgetModifiersSets.currentIndex())],
            ) = (
                dict(self.modifiers_sets_dict[str(self.tabWidgetModifiersSets.currentIndex())]),
                dict(self.modifiers_sets_dict[str(self.tabWidgetModifiersSets.currentIndex() - 1)]),
            )
            self.tabWidgetModifiersSets.setCurrentIndex(self.tabWidgetModifiersSets.currentIndex() - 1)
            self.tabMem = self.tabWidgetModifiersSets.currentIndex()

    def moveSetRight(self):
        """
        move selected modifiers set right
        """
        if self.tabWidgetModifiersSets.currentIndex() < self.tabWidgetModifiersSets.count() - 1:
            (
                self.modifiers_sets_dict[str(self.tabWidgetModifiersSets.currentIndex() + 1)],
                self.modifiers_sets_dict[str(self.tabWidgetModifiersSets.currentIndex())],
            ) = (
                dict(self.modifiers_sets_dict[str(self.tabWidgetModifiersSets.currentIndex())]),
                dict(self.modifiers_sets_dict[str(self.tabWidgetModifiersSets.currentIndex() + 1)]),
            )

            self.tabWidgetModifiersSets.setCurrentIndex(self.tabWidgetModifiersSets.currentIndex() + 1)
            self.tabMem = self.tabWidgetModifiersSets.currentIndex()

    def moveModifierUp(self):
        """
        move up the selected modifier
        """
        if self.lwModifiers.currentRow() >= 0:
            currentRow = self.lwModifiers.currentRow()
            currentItem = self.lwModifiers.takeItem(currentRow)
            self.lwModifiers.insertItem(currentRow - 1, currentItem)
            self.lwModifiers.setCurrentItem(currentItem)
            self.modifiers_sets_dict[str(self.tabWidgetModifiersSets.currentIndex())]["values"] = [
                self.lwModifiers.item(x).text() for x in range(self.lwModifiers.count())
            ]

    def moveModifierDown(self):
        """
        move down the selected modifier
        """
        if self.lwModifiers.currentRow() >= 0:
            currentRow = self.lwModifiers.currentRow()
            currentItem = self.lwModifiers.takeItem(currentRow)
            self.lwModifiers.insertItem(currentRow + 1, currentItem)
            self.lwModifiers.setCurrentItem(currentItem)
            self.modifiers_sets_dict[str(self.tabWidgetModifiersSets.currentIndex())]["values"] = [
                self.lwModifiers.item(x).text() for x in range(self.lwModifiers.count())
            ]

    def addSet(self):
        """
        Add a set of modifiers
        """

        # no modifiers set
        if self.tabWidgetModifiersSets.currentIndex() == -1:
            self.modifiers_sets_dict[str(len(self.modifiers_sets_dict))] = {
                "name": "",
                "description": "",
                "type": cfg.SINGLE_SELECTION,
                "values": [],
            }
            self.tabWidgetModifiersSets.addTab(QWidget(), f"Set #{len(self.modifiers_sets_dict)}")
            self.tabWidgetModifiersSets.setCurrentIndex(self.tabWidgetModifiersSets.count() - 1)
            self.tabMem = self.tabWidgetModifiersSets.currentIndex()

            # set visible and available buttons and others elements
            for w in (
                self.lb_name,
                self.lbType,
                self.lbValues,
                self.le_name,
                self.lb_description,
                self.le_description,
                self.cbType,
                self.lwModifiers,
                self.pbMoveUp,
                self.pbMoveDown,
                self.pbRemoveModifier,
                self.pbRemoveSet,
                self.pbMoveSetLeft,
                self.pbMoveSetRight,
                self.pb_add_subjects,
                self.pb_load_file,
                self.pb_sort_modifiers,
            ):
                w.setVisible(True)
            for w in (self.leModifier, self.leCode, self.pbAddModifier, self.pbModifyModifier):
                w.setEnabled(True)
            return

        if len(self.modifiers_sets_dict[str(self.tabWidgetModifiersSets.currentIndex())]):
            self.modifiers_sets_dict[str(len(self.modifiers_sets_dict))] = {
                "name": "",
                "description": "",
                "type": cfg.SINGLE_SELECTION,
                "values": [],
            }
            self.tabWidgetModifiersSets.addTab(QWidget(), f"Set #{len(self.modifiers_sets_dict)}")
            self.tabWidgetModifiersSets.setCurrentIndex(self.tabWidgetModifiersSets.count() - 1)
            self.tabMem = self.tabWidgetModifiersSets.currentIndex()

        else:
            QMessageBox.information(
                self,
                cfg.programName,
                "It is not possible to add a modifiers' set while the current modifiers' set is empty.",
            )

    def removeSet(self):
        """
        remove set of modifiers
        """

        if self.tabWidgetModifiersSets.currentIndex() != -1:
            if dialog.MessageDialog(cfg.programName, "Confirm deletion of this set of modifiers?", [cfg.YES, cfg.NO]) == cfg.YES:
                index_to_delete = self.tabWidgetModifiersSets.currentIndex()

                for k in range(index_to_delete, len(self.modifiers_sets_dict) - 1):
                    self.modifiers_sets_dict[str(k)] = self.modifiers_sets_dict[str(k + 1)]
                # del last key
                del self.modifiers_sets_dict[str(len(self.modifiers_sets_dict) - 1)]

                # remove all tabs
                while self.tabWidgetModifiersSets.count():
                    self.tabWidgetModifiersSets.removeTab(0)

                # recreate tabs
                for idx in sorted_keys(self.modifiers_sets_dict):
                    self.tabWidgetModifiersSets.addTab(QWidget(), f"Set #{int(idx) + 1}")

                # set not visible and not available buttons and others elements
                if self.tabWidgetModifiersSets.currentIndex() == -1:
                    for w in (
                        self.lb_name,
                        self.le_name,
                        self.lbType,
                        self.lbValues,
                        self.lb_description,
                        self.le_description,
                        self.cbType,
                        self.lwModifiers,
                        self.pbMoveUp,
                        self.pbMoveDown,
                        self.pbRemoveModifier,
                        self.pbRemoveSet,
                        self.pbMoveSetLeft,
                        self.pbMoveSetRight,
                    ):
                        w.setVisible(False)
                    for w in [self.leModifier, self.leCode, self.pbAddModifier, self.pbModifyModifier]:
                        w.setEnabled(False)

                if not len(self.modifiers_sets_dict):
                    # set invisible and unavailable buttons and others elements
                    for w in [
                        self.lb_name,
                        self.le_name,
                        self.lbType,
                        self.lbValues,
                        self.lb_description,
                        self.le_description,
                        self.cbType,
                        self.lwModifiers,
                        self.pbMoveUp,
                        self.pbMoveDown,
                        self.pbRemoveModifier,
                        self.pbRemoveSet,
                        self.pbMoveSetLeft,
                        self.pbMoveSetRight,
                        self.pb_add_subjects,
                        self.pb_load_file,
                        self.pb_sort_modifiers,
                    ]:
                        w.setVisible(False)
                    for w in [self.leModifier, self.leCode, self.pbAddModifier, self.pbModifyModifier]:
                        w.setEnabled(False)
                    return

        else:
            QMessageBox.information(self, cfg.programName, "It is not possible to remove the last modifiers' set.")

    def modifyModifier(self):
        """
        modify modifier <- arrow
        """

        if self.lwModifiers.currentRow() >= 0:
            txt = self.lwModifiers.currentItem().text()
            code = ""
            if "(" in txt and ")" in txt:
                code = txt.split("(")[1].split(")")[0]

            self.leModifier.setText(txt.split("(")[0].strip())
            self.leCode.setText(code)

            self.modifiers_sets_dict[str(self.tabWidgetModifiersSets.currentIndex())]["values"].remove(
                self.lwModifiers.currentItem().text()
            )

            self.itemPositionMem = self.lwModifiers.currentRow()
            self.lwModifiers.takeItem(self.lwModifiers.currentRow())
        else:
            QMessageBox.information(self, cfg.programName, "Select a modifier to modify from the modifiers set")

    def removeModifier(self):
        """
        remove modifier from set
        """
        if self.lwModifiers.currentIndex().row() >= 0:
            self.lwModifiers.takeItem(self.lwModifiers.currentIndex().row())
            self.modifiers_sets_dict[str(self.tabWidgetModifiersSets.currentIndex())]["values"] = [
                self.lwModifiers.item(x).text() for x in range(self.lwModifiers.count())
            ]

    def addModifier(self):
        """
        add a modifier to set
        """

        txt = self.leModifier.text().strip()
        for c in cfg.CHAR_FORBIDDEN_IN_MODIFIERS:
            if c in txt:
                QMessageBox.critical(
                    self,
                    cfg.programName,
                    (
                        f"The character <b>{c}</b> is not allowed.<br>"
                        "The following characters are not allowed in modifiers:<br>"
                        f"<b>{cfg.CHAR_FORBIDDEN_IN_MODIFIERS}</b>"
                    ),
                )
                self.leModifier.setFocus()
                return

        if txt:
            if txt in [self.lwModifiers.item(x).text() for x in range(self.lwModifiers.count())]:
                QMessageBox.critical(self, cfg.programName, f"The modifier <b>{txt}</b> is already in the list")
                return

            if not self.modifiers_sets_dict:
                self.modifiers_sets_dict["0"] = {
                    "name": "",
                    "description": "",
                    "type": cfg.SINGLE_SELECTION,
                    "values": [],
                }

            if len(self.leCode.text().strip()) > 1:
                if self.leCode.text().strip().upper() not in cfg.function_keys.values():
                    QMessageBox.critical(
                        self,
                        cfg.programName,
                        "The modifier key code can not exceed one key\nSelect one key or a function key (F1, F2 ... F12)",
                    )
                    self.leCode.setFocus()
                    return

            if self.leCode.text().strip():
                for c in cfg.CHAR_FORBIDDEN_IN_MODIFIERS:
                    if c in self.leCode.text().strip():
                        QMessageBox.critical(
                            self,
                            cfg.programName,
                            f"The modifier key code is not allowed {cfg.CHAR_FORBIDDEN_IN_MODIFIERS}!",
                        )
                        self.leCode.setFocus()
                        return

                # check if code already exists
                if not self.modifiers_sets_dict:
                    self.modifiers_sets_dict[str(self.tabWidgetModifiersSets.currentIndex())] = {
                        "name": "",
                        "description": "",
                        "type": cfg.SINGLE_SELECTION,
                        "values": [],
                    }

                if "(" + self.leCode.text().strip() + ")" in " ".join(
                    self.modifiers_sets_dict[str(self.tabWidgetModifiersSets.currentIndex())]["values"]
                ):
                    QMessageBox.critical(self, cfg.programName, f"The shortcut code <b>{self.leCode.text().strip()}</b> already exists!")
                    self.leCode.setFocus()
                    return
                txt += f" ({self.leCode.text().strip()})"

            if self.itemPositionMem != -1:
                self.lwModifiers.insertItem(self.itemPositionMem, txt)
            else:
                self.lwModifiers.addItem(txt)

            self.modifiers_sets_dict[str(self.tabWidgetModifiersSets.currentIndex())]["values"] = [
                self.lwModifiers.item(x).text() for x in range(self.lwModifiers.count())
            ]
            self.leModifier.setText("")
            self.leCode.setText("")

        else:
            QMessageBox.critical(self, cfg.programName, "No modifier to add!")
            self.leModifier.setFocus()

    def tabWidgetModifiersSets_changed(self, tabIndex):
        """
        user changed the tab widget
        """

        # check if modifier field empty
        if self.leModifier.text() and tabIndex != self.tabMem:
            if (
                dialog.MessageDialog(
                    cfg.programName,
                    (
                        "You are working on a behavior.<br>"
                        "If you change the modifier's set it will be lost.<br>"
                        "Do you want to change modifiers set"
                    ),
                    [cfg.YES, cfg.NO],
                )
                == cfg.NO
            ):
                self.tabWidgetModifiersSets.setCurrentIndex(self.tabMem)
                return

        if tabIndex != self.tabMem:
            self.lwModifiers.clear()
            self.leCode.clear()
            self.leModifier.clear()

            self.tabMem = tabIndex

            if tabIndex != -1:
                self.le_name.setText(self.modifiers_sets_dict[str(tabIndex)]["name"])
                self.le_description.setText(self.modifiers_sets_dict[str(tabIndex)].get("description", ""))
                self.cbType.setCurrentIndex(self.modifiers_sets_dict[str(tabIndex)]["type"])
                self.lwModifiers.addItems(self.modifiers_sets_dict[str(tabIndex)]["values"])

    def getModifiers(self) -> str:
        """
        returns modifiers as string
        """
        keys_to_delete = []
        for idx in self.modifiers_sets_dict:
            if (
                self.modifiers_sets_dict[idx]["type"] in (cfg.SINGLE_SELECTION, cfg.MULTI_SELECTION)
                and not self.modifiers_sets_dict[idx]["values"]
            ):
                keys_to_delete.append(idx)

        for idx in keys_to_delete:
            del self.modifiers_sets_dict[idx]

        return json.dumps(self.modifiers_sets_dict) if self.modifiers_sets_dict else ""



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

"""Example of qdarkstyle use for Python and Qt applications.

This module a main window with every item that could be created with
Qt Design (common ones) in the basic states (enabled/disabled), and
(checked/unchecked) for those who has this attribute.

Requirements:

    - Python 3
    - QtPy
    - PyQt6 or PyQt5 or PyQt4 or PySide6 or PySide2 or PySide
    - PyQtGraph or Qt.Py (if chosen)

To run this example using PyQt5, simple do

.. code-block:: python

    python example.py

or

.. code-block:: python

    python example.py  --qt_from=pyqt5

Other options for qt_from are: pyqt6, pyqt5, pyside6, pyside2, pyqt, pyside,
qtpy, pyqtgraph, and qt.py.

Also, you can run the example without any theme (none), to check for problems.

.. code-block:: python

    python example.py  --qt_from=pyqt5 --palette=none

Note:
    qdarkstyle does not have to be installed to run the example.

"""

# Standard library imports

import argparse
import logging
import os
import sys
import platform
import time

# Make the example runnable without the need to install and include ui
sys.path.insert(0, os.path.abspath(os.path.dirname(os.path.abspath(__file__)) + '/../..'))
sys.path.insert(0, os.path.abspath(os.path.dirname(os.path.abspath(__file__)) + '/../ui'))

# Must be in this place, after setting path, to not need to install
import qdarkstyle  # noqa: E402
from qdarkstyle.dark.palette import DarkPalette  # noqa: E402
from qdarkstyle.light.palette import LightPalette  # noqa: E402

# Set log for debug
logging.basicConfig(level=logging.INFO)
_logger = logging.getLogger(__name__)

here = os.path.abspath(os.path.dirname(__file__))

# Constants
SCREENSHOTS_PATH = qdarkstyle.IMAGES_PATH


def main():
    """Execute QDarkStyle example."""
    parser = argparse.ArgumentParser(description=__doc__,
                                     formatter_class=argparse.RawDescriptionHelpFormatter)
    parser.add_argument('--palette', default='dark', type=str,
                        choices=['dark', 'light', 'none'],
                        help="Palette to display. Using 'none' there is no style sheet applied, OS like.")
    parser.add_argument('--qt_from', default='pyqt5', type=str,
                        choices=['pyqt6', 'pyqt5', 'pyqt', 'pyside6', 'pyside2', 'pyside', 'qtpy', 'pyqtgraph', 'qt.py'],
                        help="Choose which binding and/or abstraction is to be used to run the example. Default is 'qtpy'")
    parser.add_argument('--test', action='store_true',
                        help="Auto close window after 2s.")
    parser.add_argument('--screenshots', action='store_true',
                        help="Generate screenshots on images folder.")
    parser.add_argument('--offscreen', action='store_true',
                        help="Do not try to show the screen (running on server).")
    parser.add_argument('--reset', action='store_true',
                        help="Reset GUI settings (position, size) then opens.")

    # Parsing arguments from command line
    args = parser.parse_args()

    # To avoid problems when testing without screen
    if args.test or args.offscreen:
        os.environ['QT_QPA_PLATFORM'] = 'offscreen'

    # Set QT_API variable before importing QtPy
    if args.qt_from in ['pyqt', 'pyqt5', 'pyqt6', 'pyside', 'pyside2', 'pyside6']:
        os.environ['QT_API'] = args.qt_from
    elif args.qt_from == 'pyqtgraph':
        os.environ['QT_API'] = os.environ['PYQTGRAPH_QT_LIB']
    elif args.qt_from in ['qt.py', 'qt']:
        try:
            import Qt
        except ImportError:
            print('Could not import Qt (Qt.Py)')
        else:
            os.environ['QT_API'] = Qt.__binding__

    # QtPy imports
    from qtpy import API_NAME, QT_VERSION, PYQT_VERSION, PYSIDE_VERSION, uic
    from qtpy import __version__ as QTPY_VERSION
    from qtpy import QtCore, QtGui, QtWidgets

    # Set API_VERSION variable
    API_VERSION = ''

    if PYQT_VERSION:
        API_VERSION = PYQT_VERSION
    elif PYSIDE_VERSION:
        API_VERSION = PYSIDE_VERSION
    else:
        API_VERSION = 'Not found'

    # create the application
    app = QtWidgets.QApplication(sys.argv)
    app.setOrganizationName('QDarkStyle')
    app.setApplicationName('QDarkStyle Example')

    style = ''

    if args.palette == 'dark':
        style = qdarkstyle.load_stylesheet(palette=DarkPalette)
    elif args.palette == 'light':
        style = qdarkstyle.load_stylesheet(palette=LightPalette)

    app.setStyleSheet(style)

    # create main window
    window = QtWidgets.QMainWindow()
    window.setObjectName('mainwindow')
    uic.loadUi(os.path.join(here, 'ui/mw_menus.ui'), window)

    title = ("QDarkStyle Example - ("
             + f"Palette={args.palette}, "
             + f"QDarkStyle=v{qdarkstyle.__version__}, "
             + f"QtPy=v{QTPY_VERSION}, "
             + f"{API_NAME}=v{API_VERSION}, "
             + f"Qt=v{QT_VERSION}, "
             + f"Python=v{platform.python_version()}, "
             # Operating system info are maybe too much,
             # but different OS add info in different places
             + f"System={platform.system()}, "
             + f"Release={platform.release()}, "
             + f"Version={platform.version()}, "
             + f"Platform={platform.platform()}"
             + ")")

    _logger.info(title)
    window.setWindowTitle(title)

    # Create docks for buttons
    dw_buttons = QtWidgets.QDockWidget()
    dw_buttons.setObjectName('buttons')
    uic.loadUi(os.path.join(here, 'ui/dw_buttons.ui'), dw_buttons)
    window.addDockWidget(QtCore.Qt.RightDockWidgetArea, dw_buttons)

    # Set state indeterminate #275
    dw_buttons.checkBoxTristate.stateChanged.connect(dw_buttons.checkBoxTristateDis.setCheckState)
    dw_buttons.checkBoxTristate.setCheckState(QtCore.Qt.PartiallyChecked)

    # Add actions on popup toolbuttons
    menu = QtWidgets.QMenu()

    for action in ['Action A', 'Action B', 'Action C']:
        menu.addAction(action)

    # Add menu in special tool buttons
    dw_buttons.toolButtonDelayedPopup.setMenu(menu)
    dw_buttons.toolButtonInstantPopup.setMenu(menu)
    dw_buttons.toolButtonMenuButtonPopup.setMenu(menu)

    # Add menu in toolbar #251
    action_menu = QtWidgets.QAction(u'Menu action', window.toolBarMenus)
    action_menu.setMenu(menu)
    window.toolBarMenus.addAction(action_menu)

    # Add color to tab title text #212
    window.tabWidget.tabBar().setTabTextColor(3, QtGui.QColor('red'))

    # Connect dialogs to buttons
    window.toolButtonColorDialog.clicked.connect(lambda: QtWidgets.QColorDialog().exec())
    window.toolButtonFileDialog.clicked.connect(lambda: QtWidgets.QFileDialog().exec())
    window.toolButtonFileDialogStatic.clicked.connect(lambda: QtWidgets.QFileDialog.getOpenFileNames())
    window.toolButtonFontDialog.clicked.connect(lambda: QtWidgets.QFontDialog().exec())
    window.toolButtonInputDialog.clicked.connect(lambda: QtWidgets.QInputDialog().exec())
    window.toolButtonMessageBox.clicked.connect(lambda: QtWidgets.QMessageBox().exec())
    window.toolButtonMessageBoxStatic.clicked.connect(lambda: QtWidgets.QMessageBox.critical(window, "Critical title", "Critical message"))
    window.toolButtonProgressDialog.clicked.connect(lambda: QtWidgets.QProgressDialog().exec())

    # Create docks for buttons
    dw_displays = QtWidgets.QDockWidget()
    dw_displays.setObjectName('displays')
    uic.loadUi(os.path.join(here, 'ui/dw_displays.ui'), dw_displays)
    window.addDockWidget(QtCore.Qt.RightDockWidgetArea, dw_displays)

    # Create docks for inputs - no fields
    dw_inputs_no_fields = QtWidgets.QDockWidget()
    dw_inputs_no_fields.setObjectName('inputs_no_fields')
    uic.loadUi(os.path.join(here, 'ui/dw_inputs_no_fields.ui'), dw_inputs_no_fields)
    window.addDockWidget(QtCore.Qt.RightDockWidgetArea, dw_inputs_no_fields)

    # Create docks for inputs - fields
    dw_inputs_fields = QtWidgets.QDockWidget()
    dw_inputs_fields.setObjectName('inputs_fields')
    uic.loadUi(os.path.join(here, 'ui/dw_inputs_fields.ui'), dw_inputs_fields)
    window.addDockWidget(QtCore.Qt.RightDockWidgetArea, dw_inputs_fields)

    # Create docks for widgets
    dw_widgets = QtWidgets.QDockWidget()
    dw_widgets.setObjectName('widgets')
    uic.loadUi(os.path.join(here, 'ui/dw_widgets.ui'), dw_widgets)
    window.addDockWidget(QtCore.Qt.LeftDockWidgetArea, dw_widgets)

    # Create docks for views
    dw_views = QtWidgets.QDockWidget()
    dw_views.setObjectName('views')
    uic.loadUi(os.path.join(here, 'ui/dw_views.ui'), dw_views)
    window.addDockWidget(QtCore.Qt.LeftDockWidgetArea, dw_views)

    # Create docks for containers - no tabs
    dw_containers_no_tabs = QtWidgets.QDockWidget()
    dw_containers_no_tabs.setObjectName('containers_no_tabs')
    uic.loadUi(os.path.join(here, 'ui/dw_containers_no_tabs.ui'), dw_containers_no_tabs)
    window.addDockWidget(QtCore.Qt.LeftDockWidgetArea, dw_containers_no_tabs)

    # Create docks for containters - tabs
    dw_containers_tabs = QtWidgets.QDockWidget()
    dw_containers_tabs.setObjectName('containers_tabs')
    uic.loadUi(os.path.join(here, 'ui/dw_containers_tabs.ui'), dw_containers_tabs)
    window.addDockWidget(QtCore.Qt.LeftDockWidgetArea, dw_containers_tabs)

    # Tabify right docks
    window.tabifyDockWidget(dw_buttons, dw_displays)
    window.tabifyDockWidget(dw_displays, dw_inputs_fields)
    window.tabifyDockWidget(dw_inputs_fields, dw_inputs_no_fields)

    # Tabify left docks
    window.tabifyDockWidget(dw_containers_no_tabs, dw_containers_tabs)
    window.tabifyDockWidget(dw_containers_tabs, dw_widgets)
    window.tabifyDockWidget(dw_widgets, dw_views)

    # Issues #9120, #9121 on Spyder
    qstatusbar = QtWidgets.QStatusBar()
    qstatusbar.addWidget(QtWidgets.QLabel('Issue Spyder #9120, #9121 - background not matching.'))
    qstatusbar.addWidget(QtWidgets.QPushButton('OK'))

    # Add info also in status bar for screenshots get it
    qstatusbar.addWidget(QtWidgets.QLabel('INFO: ' + title))
    window.setStatusBar(qstatusbar)

    # Todo: add report info and other info in HELP graphical

    # Auto quit after 2s when in test mode
    if args.test:
        QtCore.QTimer.singleShot(2000, app.exit)

    # Save screenshots for different displays and quit
    if args.screenshots:
        window.showFullScreen()
        create_screenshots(app, window, args)
    # Do not read settings when taking screenshots - like reset
    else:
        _read_settings(window, args.reset, QtCore.QSettings)
        window.showMaximized()

    app.exec_()
    _write_settings(window, QtCore.QSettings)


def _write_settings(window, QSettingsClass):
    """Get window settings and write it into a file."""
    settings = QSettingsClass('QDarkStyle', 'QDarkStyle Example')
    settings.setValue('pos', window.pos())
    settings.setValue('size', window.size())
    settings.setValue('state', window.saveState())


def _read_settings(window, reset, QSettingsClass):
    """Read and set window settings from a file."""
    settings = QSettingsClass('QDarkStyle', 'QDarkStyle Example')

    try:
        pos = settings.value('pos', window.pos())
        size = settings.value('size', window.size())
        state = settings.value('state', window.saveState())
    except Exception:
        pos = settings.value('pos', window.pos(), type='QPoint')
        size = settings.value('size', window.size(), type='QSize')
        state = settings.value('state', window.saveState(), type='QByteArray')

    if not reset:
        window.restoreState(state)
        window.resize(size)
        window.move(pos)


def create_screenshots(app, window, args):
    """Save screenshots for different application views and quit."""

    theme = args.palette

    print('\nCreating {} screenshots'.format(theme))

    docks = window.findChildren(QtWidgets.QDockWidget)
    tabs = window.findChildren(QtWidgets.QTabWidget)

    widget_data = {
        'containers_no_tabs_buttons.png': [
            'Containers - No Tabs',
            'Buttons',
        ],
        'containers_tabs_displays.png': [
            'Containers - Tabs',
            'Displays',
        ],
        'widgets_inputs_fields.png': [
            'Widgets',
            'Inputs - Fields',
        ],
        'views_inputs_no_fields.png': [
            'Views',
            'Inputs - No Fields',
        ]
    }

    # Central widget tabs of with examples, reset positions
    tab = [tab for tab in tabs if tab.count() >= 12][0]
    tab.setCurrentIndex(0)

    QtCore.QCoreApplication.processEvents()

    for fname_suffix, dw_titles in widget_data.items():
        png_path = os.path.join(SCREENSHOTS_PATH, theme, fname_suffix)
        print('\t' + png_path)

        for dw in docks:
            if dw.windowTitle() in dw_titles:
                print('Evidencing : ', dw.windowTitle())
                dw.raise_()
                dw.show()
                QtCore.QCoreApplication.processEvents()

        # Attention: any change in update, processEvent and sleep calls
        # make those screenshots not working, specially the first one.
        # It seems that processEvents are not working properly

        window.update()
        window.showFullScreen()
        QtCore.QCoreApplication.processEvents()

        time.sleep(0.5)
        QtCore.QCoreApplication.processEvents()

        screen = QtGui.QGuiApplication.primaryScreen()
        QtCore.QCoreApplication.processEvents()
        pixmap = screen.grabWindow(window.winId())

        # Yeah, this is duplicated to avoid screenshot problems
        screen = QtGui.QGuiApplication.primaryScreen()
        QtCore.QCoreApplication.processEvents()
        pixmap = screen.grabWindow(window.winId())

        img = pixmap.toImage()
        img.save(png_path)

        QtCore.QCoreApplication.processEvents()

    QtCore.QCoreApplication.processEvents()
    window.close()
    print('\n')
    app.exit(sys.exit())


if __name__ == "__main__":
    sys.exit(main())



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'boris/project.ui'
#
# Created by: PyQt5 UI code generator 5.15.9
#
# WARNING: Any manual changes made to this file will be lost when pyuic5 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_dlgProject(object):
    def setupUi(self, dlgProject):
        dlgProject.setObjectName("dlgProject")
        dlgProject.resize(1202, 697)
        self.verticalLayout_7 = QtWidgets.QVBoxLayout(dlgProject)
        self.verticalLayout_7.setObjectName("verticalLayout_7")
        self.verticalLayout_6 = QtWidgets.QVBoxLayout()
        self.verticalLayout_6.setObjectName("verticalLayout_6")
        self.tabProject = QtWidgets.QTabWidget(dlgProject)
        self.tabProject.setObjectName("tabProject")
        self.tabInformation = QtWidgets.QWidget()
        self.tabInformation.setObjectName("tabInformation")
        self.verticalLayout = QtWidgets.QVBoxLayout(self.tabInformation)
        self.verticalLayout.setObjectName("verticalLayout")
        self.horizontalLayout_15 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_15.setObjectName("horizontalLayout_15")
        self.label = QtWidgets.QLabel(self.tabInformation)
        self.label.setObjectName("label")
        self.horizontalLayout_15.addWidget(self.label)
        self.leProjectName = QtWidgets.QLineEdit(self.tabInformation)
        self.leProjectName.setObjectName("leProjectName")
        self.horizontalLayout_15.addWidget(self.leProjectName)
        self.verticalLayout.addLayout(self.horizontalLayout_15)
        self.lbProjectFilePath = QtWidgets.QLabel(self.tabInformation)
        self.lbProjectFilePath.setObjectName("lbProjectFilePath")
        self.verticalLayout.addWidget(self.lbProjectFilePath)
        self.horizontalLayout_18 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_18.setObjectName("horizontalLayout_18")
        self.label_7 = QtWidgets.QLabel(self.tabInformation)
        self.label_7.setObjectName("label_7")
        self.horizontalLayout_18.addWidget(self.label_7)
        self.dteDate = QtWidgets.QDateTimeEdit(self.tabInformation)
        self.dteDate.setCalendarPopup(True)
        self.dteDate.setObjectName("dteDate")
        self.horizontalLayout_18.addWidget(self.dteDate)
        spacerItem = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_18.addItem(spacerItem)
        self.verticalLayout.addLayout(self.horizontalLayout_18)
        self.label_6 = QtWidgets.QLabel(self.tabInformation)
        self.label_6.setAlignment(QtCore.Qt.AlignLeading|QtCore.Qt.AlignLeft|QtCore.Qt.AlignTop)
        self.label_6.setObjectName("label_6")
        self.verticalLayout.addWidget(self.label_6)
        self.teDescription = QtWidgets.QPlainTextEdit(self.tabInformation)
        self.teDescription.setObjectName("teDescription")
        self.verticalLayout.addWidget(self.teDescription)
        self.horizontalLayout_19 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_19.setObjectName("horizontalLayout_19")
        self.lbTimeFormat = QtWidgets.QLabel(self.tabInformation)
        self.lbTimeFormat.setObjectName("lbTimeFormat")
        self.horizontalLayout_19.addWidget(self.lbTimeFormat)
        self.rbSeconds = QtWidgets.QRadioButton(self.tabInformation)
        self.rbSeconds.setChecked(True)
        self.rbSeconds.setObjectName("rbSeconds")
        self.horizontalLayout_19.addWidget(self.rbSeconds)
        self.rbHMS = QtWidgets.QRadioButton(self.tabInformation)
        self.rbHMS.setObjectName("rbHMS")
        self.horizontalLayout_19.addWidget(self.rbHMS)
        spacerItem1 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_19.addItem(spacerItem1)
        self.verticalLayout.addLayout(self.horizontalLayout_19)
        self.lb_project_format_version = QtWidgets.QLabel(self.tabInformation)
        self.lb_project_format_version.setObjectName("lb_project_format_version")
        self.verticalLayout.addWidget(self.lb_project_format_version)
        self.tabProject.addTab(self.tabInformation, "")
        self.tabEthogram = QtWidgets.QWidget()
        self.tabEthogram.setObjectName("tabEthogram")
        self.verticalLayout_10 = QtWidgets.QVBoxLayout(self.tabEthogram)
        self.verticalLayout_10.setObjectName("verticalLayout_10")
        self.verticalLayout_5 = QtWidgets.QVBoxLayout()
        self.verticalLayout_5.setObjectName("verticalLayout_5")
        self.horizontalLayout_11 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_11.setObjectName("horizontalLayout_11")
        self.twBehaviors = QtWidgets.QTableWidget(self.tabEthogram)
        self.twBehaviors.setAutoFillBackground(False)
        self.twBehaviors.setFrameShadow(QtWidgets.QFrame.Sunken)
        self.twBehaviors.setMidLineWidth(0)
        self.twBehaviors.setAlternatingRowColors(True)
        self.twBehaviors.setSelectionMode(QtWidgets.QAbstractItemView.SingleSelection)
        self.twBehaviors.setSelectionBehavior(QtWidgets.QAbstractItemView.SelectRows)
        self.twBehaviors.setObjectName("twBehaviors")
        self.twBehaviors.setColumnCount(9)
        self.twBehaviors.setRowCount(0)
        item = QtWidgets.QTableWidgetItem()
        self.twBehaviors.setHorizontalHeaderItem(0, item)
        item = QtWidgets.QTableWidgetItem()
        self.twBehaviors.setHorizontalHeaderItem(1, item)
        item = QtWidgets.QTableWidgetItem()
        self.twBehaviors.setHorizontalHeaderItem(2, item)
        item = QtWidgets.QTableWidgetItem()
        self.twBehaviors.setHorizontalHeaderItem(3, item)
        item = QtWidgets.QTableWidgetItem()
        self.twBehaviors.setHorizontalHeaderItem(4, item)
        item = QtWidgets.QTableWidgetItem()
        self.twBehaviors.setHorizontalHeaderItem(5, item)
        item = QtWidgets.QTableWidgetItem()
        self.twBehaviors.setHorizontalHeaderItem(6, item)
        item = QtWidgets.QTableWidgetItem()
        self.twBehaviors.setHorizontalHeaderItem(7, item)
        item = QtWidgets.QTableWidgetItem()
        self.twBehaviors.setHorizontalHeaderItem(8, item)
        self.twBehaviors.horizontalHeader().setSortIndicatorShown(False)
        self.twBehaviors.verticalHeader().setSortIndicatorShown(False)
        self.horizontalLayout_11.addWidget(self.twBehaviors)
        self.verticalLayout_11 = QtWidgets.QVBoxLayout()
        self.verticalLayout_11.setObjectName("verticalLayout_11")
        self.pb_behavior = QtWidgets.QPushButton(self.tabEthogram)
        self.pb_behavior.setObjectName("pb_behavior")
        self.verticalLayout_11.addWidget(self.pb_behavior)
        self.pb_import = QtWidgets.QPushButton(self.tabEthogram)
        self.pb_import.setObjectName("pb_import")
        self.verticalLayout_11.addWidget(self.pb_import)
        self.pbBehaviorsCategories = QtWidgets.QPushButton(self.tabEthogram)
        self.pbBehaviorsCategories.setObjectName("pbBehaviorsCategories")
        self.verticalLayout_11.addWidget(self.pbBehaviorsCategories)
        spacerItem2 = QtWidgets.QSpacerItem(20, 40, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.verticalLayout_11.addItem(spacerItem2)
        self.pb_exclusion_matrix = QtWidgets.QPushButton(self.tabEthogram)
        self.pb_exclusion_matrix.setObjectName("pb_exclusion_matrix")
        self.verticalLayout_11.addWidget(self.pb_exclusion_matrix)
        self.pbExportEthogram = QtWidgets.QPushButton(self.tabEthogram)
        self.pbExportEthogram.setObjectName("pbExportEthogram")
        self.verticalLayout_11.addWidget(self.pbExportEthogram)
        self.horizontalLayout_11.addLayout(self.verticalLayout_11)
        self.verticalLayout_5.addLayout(self.horizontalLayout_11)
        self.lbObservationsState = QtWidgets.QLabel(self.tabEthogram)
        self.lbObservationsState.setObjectName("lbObservationsState")
        self.verticalLayout_5.addWidget(self.lbObservationsState)
        self.verticalLayout_10.addLayout(self.verticalLayout_5)
        self.tabProject.addTab(self.tabEthogram, "")
        self.tabSubjects = QtWidgets.QWidget()
        self.tabSubjects.setObjectName("tabSubjects")
        self.verticalLayout_16 = QtWidgets.QVBoxLayout(self.tabSubjects)
        self.verticalLayout_16.setObjectName("verticalLayout_16")
        self.verticalLayout_14 = QtWidgets.QVBoxLayout()
        self.verticalLayout_14.setObjectName("verticalLayout_14")
        self.horizontalLayout_12 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_12.setObjectName("horizontalLayout_12")
        self.twSubjects = QtWidgets.QTableWidget(self.tabSubjects)
        self.twSubjects.setAutoFillBackground(False)
        self.twSubjects.setFrameShadow(QtWidgets.QFrame.Sunken)
        self.twSubjects.setMidLineWidth(0)
        self.twSubjects.setSelectionMode(QtWidgets.QAbstractItemView.SingleSelection)
        self.twSubjects.setSelectionBehavior(QtWidgets.QAbstractItemView.SelectRows)
        self.twSubjects.setObjectName("twSubjects")
        self.twSubjects.setColumnCount(3)
        self.twSubjects.setRowCount(0)
        item = QtWidgets.QTableWidgetItem()
        self.twSubjects.setHorizontalHeaderItem(0, item)
        item = QtWidgets.QTableWidgetItem()
        self.twSubjects.setHorizontalHeaderItem(1, item)
        item = QtWidgets.QTableWidgetItem()
        self.twSubjects.setHorizontalHeaderItem(2, item)
        self.horizontalLayout_12.addWidget(self.twSubjects)
        self.verticalLayout_15 = QtWidgets.QVBoxLayout()
        self.verticalLayout_15.setObjectName("verticalLayout_15")
        self.pb_subjects = QtWidgets.QPushButton(self.tabSubjects)
        self.pb_subjects.setObjectName("pb_subjects")
        self.verticalLayout_15.addWidget(self.pb_subjects)
        self.pbImportSubjectsFromProject = QtWidgets.QPushButton(self.tabSubjects)
        self.pbImportSubjectsFromProject.setObjectName("pbImportSubjectsFromProject")
        self.verticalLayout_15.addWidget(self.pbImportSubjectsFromProject)
        spacerItem3 = QtWidgets.QSpacerItem(20, 40, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.verticalLayout_15.addItem(spacerItem3)
        self.pb_export_subjects = QtWidgets.QPushButton(self.tabSubjects)
        self.pb_export_subjects.setObjectName("pb_export_subjects")
        self.verticalLayout_15.addWidget(self.pb_export_subjects)
        self.horizontalLayout_12.addLayout(self.verticalLayout_15)
        self.verticalLayout_14.addLayout(self.horizontalLayout_12)
        self.lbSubjectsState = QtWidgets.QLabel(self.tabSubjects)
        self.lbSubjectsState.setObjectName("lbSubjectsState")
        self.verticalLayout_14.addWidget(self.lbSubjectsState)
        self.verticalLayout_16.addLayout(self.verticalLayout_14)
        self.tabProject.addTab(self.tabSubjects, "")
        self.tabIndependentVariables = QtWidgets.QWidget()
        self.tabIndependentVariables.setObjectName("tabIndependentVariables")
        self.horizontalLayout_14 = QtWidgets.QHBoxLayout(self.tabIndependentVariables)
        self.horizontalLayout_14.setObjectName("horizontalLayout_14")
        self.horizontalLayout_13 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_13.setObjectName("horizontalLayout_13")
        self.verticalLayout_2 = QtWidgets.QVBoxLayout()
        self.verticalLayout_2.setObjectName("verticalLayout_2")
        self.twVariables = QtWidgets.QTableWidget(self.tabIndependentVariables)
        self.twVariables.setAutoFillBackground(False)
        self.twVariables.setFrameShadow(QtWidgets.QFrame.Sunken)
        self.twVariables.setMidLineWidth(0)
        self.twVariables.setEditTriggers(QtWidgets.QAbstractItemView.NoEditTriggers)
        self.twVariables.setDragDropOverwriteMode(False)
        self.twVariables.setAlternatingRowColors(True)
        self.twVariables.setSelectionMode(QtWidgets.QAbstractItemView.SingleSelection)
        self.twVariables.setSelectionBehavior(QtWidgets.QAbstractItemView.SelectRows)
        self.twVariables.setObjectName("twVariables")
        self.twVariables.setColumnCount(5)
        self.twVariables.setRowCount(0)
        item = QtWidgets.QTableWidgetItem()
        self.twVariables.setHorizontalHeaderItem(0, item)
        item = QtWidgets.QTableWidgetItem()
        self.twVariables.setHorizontalHeaderItem(1, item)
        item = QtWidgets.QTableWidgetItem()
        self.twVariables.setHorizontalHeaderItem(2, item)
        item = QtWidgets.QTableWidgetItem()
        self.twVariables.setHorizontalHeaderItem(3, item)
        item = QtWidgets.QTableWidgetItem()
        self.twVariables.setHorizontalHeaderItem(4, item)
        self.twVariables.horizontalHeader().setSortIndicatorShown(False)
        self.verticalLayout_2.addWidget(self.twVariables)
        self.horizontalLayout_3 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_3.setObjectName("horizontalLayout_3")
        self.label_2 = QtWidgets.QLabel(self.tabIndependentVariables)
        self.label_2.setMinimumSize(QtCore.QSize(120, 0))
        self.label_2.setObjectName("label_2")
        self.horizontalLayout_3.addWidget(self.label_2)
        self.leLabel = QtWidgets.QLineEdit(self.tabIndependentVariables)
        self.leLabel.setObjectName("leLabel")
        self.horizontalLayout_3.addWidget(self.leLabel)
        spacerItem4 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_3.addItem(spacerItem4)
        self.verticalLayout_2.addLayout(self.horizontalLayout_3)
        self.horizontalLayout_5 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_5.setObjectName("horizontalLayout_5")
        self.label_3 = QtWidgets.QLabel(self.tabIndependentVariables)
        self.label_3.setMinimumSize(QtCore.QSize(120, 0))
        self.label_3.setObjectName("label_3")
        self.horizontalLayout_5.addWidget(self.label_3)
        self.leDescription = QtWidgets.QLineEdit(self.tabIndependentVariables)
        self.leDescription.setObjectName("leDescription")
        self.horizontalLayout_5.addWidget(self.leDescription)
        spacerItem5 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_5.addItem(spacerItem5)
        self.verticalLayout_2.addLayout(self.horizontalLayout_5)
        self.horizontalLayout_6 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_6.setObjectName("horizontalLayout_6")
        self.label_8 = QtWidgets.QLabel(self.tabIndependentVariables)
        self.label_8.setMinimumSize(QtCore.QSize(120, 0))
        self.label_8.setObjectName("label_8")
        self.horizontalLayout_6.addWidget(self.label_8)
        self.cbType = QtWidgets.QComboBox(self.tabIndependentVariables)
        self.cbType.setMinimumSize(QtCore.QSize(120, 0))
        self.cbType.setObjectName("cbType")
        self.horizontalLayout_6.addWidget(self.cbType)
        spacerItem6 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_6.addItem(spacerItem6)
        self.verticalLayout_2.addLayout(self.horizontalLayout_6)
        self.horizontalLayout_7 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_7.setObjectName("horizontalLayout_7")
        self.label_4 = QtWidgets.QLabel(self.tabIndependentVariables)
        self.label_4.setMinimumSize(QtCore.QSize(120, 0))
        self.label_4.setObjectName("label_4")
        self.horizontalLayout_7.addWidget(self.label_4)
        self.lePredefined = QtWidgets.QLineEdit(self.tabIndependentVariables)
        self.lePredefined.setObjectName("lePredefined")
        self.horizontalLayout_7.addWidget(self.lePredefined)
        spacerItem7 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_7.addItem(spacerItem7)
        self.verticalLayout_2.addLayout(self.horizontalLayout_7)
        self.horizontalLayout_8 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_8.setObjectName("horizontalLayout_8")
        self.label_9 = QtWidgets.QLabel(self.tabIndependentVariables)
        self.label_9.setMinimumSize(QtCore.QSize(120, 0))
        self.label_9.setObjectName("label_9")
        self.horizontalLayout_8.addWidget(self.label_9)
        self.dte_default_date = QtWidgets.QDateTimeEdit(self.tabIndependentVariables)
        self.dte_default_date.setObjectName("dte_default_date")
        self.horizontalLayout_8.addWidget(self.dte_default_date)
        spacerItem8 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_8.addItem(spacerItem8)
        self.verticalLayout_2.addLayout(self.horizontalLayout_8)
        self.horizontalLayout_9 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_9.setObjectName("horizontalLayout_9")
        self.label_5 = QtWidgets.QLabel(self.tabIndependentVariables)
        self.label_5.setObjectName("label_5")
        self.horizontalLayout_9.addWidget(self.label_5)
        self.leSetValues = QtWidgets.QLineEdit(self.tabIndependentVariables)
        self.leSetValues.setObjectName("leSetValues")
        self.horizontalLayout_9.addWidget(self.leSetValues)
        spacerItem9 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_9.addItem(spacerItem9)
        self.verticalLayout_2.addLayout(self.horizontalLayout_9)
        self.horizontalLayout_13.addLayout(self.verticalLayout_2)
        self.verticalLayout_12 = QtWidgets.QVBoxLayout()
        self.verticalLayout_12.setObjectName("verticalLayout_12")
        self.pbAddVariable = QtWidgets.QPushButton(self.tabIndependentVariables)
        self.pbAddVariable.setObjectName("pbAddVariable")
        self.verticalLayout_12.addWidget(self.pbAddVariable)
        self.pbRemoveVariable = QtWidgets.QPushButton(self.tabIndependentVariables)
        self.pbRemoveVariable.setObjectName("pbRemoveVariable")
        self.verticalLayout_12.addWidget(self.pbRemoveVariable)
        self.pbImportVarFromProject = QtWidgets.QPushButton(self.tabIndependentVariables)
        self.pbImportVarFromProject.setObjectName("pbImportVarFromProject")
        self.verticalLayout_12.addWidget(self.pbImportVarFromProject)
        spacerItem10 = QtWidgets.QSpacerItem(20, 40, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.verticalLayout_12.addItem(spacerItem10)
        self.horizontalLayout_13.addLayout(self.verticalLayout_12)
        self.horizontalLayout_14.addLayout(self.horizontalLayout_13)
        self.tabProject.addTab(self.tabIndependentVariables, "")
        self.tabBehavCodingMap = QtWidgets.QWidget()
        self.tabBehavCodingMap.setObjectName("tabBehavCodingMap")
        self.verticalLayout_8 = QtWidgets.QVBoxLayout(self.tabBehavCodingMap)
        self.verticalLayout_8.setObjectName("verticalLayout_8")
        self.horizontalLayout = QtWidgets.QHBoxLayout()
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.twBehavCodingMap = QtWidgets.QTableWidget(self.tabBehavCodingMap)
        self.twBehavCodingMap.setAutoFillBackground(False)
        self.twBehavCodingMap.setFrameShadow(QtWidgets.QFrame.Sunken)
        self.twBehavCodingMap.setMidLineWidth(0)
        self.twBehavCodingMap.setSelectionMode(QtWidgets.QAbstractItemView.SingleSelection)
        self.twBehavCodingMap.setSelectionBehavior(QtWidgets.QAbstractItemView.SelectRows)
        self.twBehavCodingMap.setObjectName("twBehavCodingMap")
        self.twBehavCodingMap.setColumnCount(2)
        self.twBehavCodingMap.setRowCount(0)
        item = QtWidgets.QTableWidgetItem()
        self.twBehavCodingMap.setHorizontalHeaderItem(0, item)
        item = QtWidgets.QTableWidgetItem()
        self.twBehavCodingMap.setHorizontalHeaderItem(1, item)
        self.horizontalLayout.addWidget(self.twBehavCodingMap)
        self.verticalLayout_4 = QtWidgets.QVBoxLayout()
        self.verticalLayout_4.setObjectName("verticalLayout_4")
        self.pbAddBehaviorsCodingMap = QtWidgets.QPushButton(self.tabBehavCodingMap)
        self.pbAddBehaviorsCodingMap.setObjectName("pbAddBehaviorsCodingMap")
        self.verticalLayout_4.addWidget(self.pbAddBehaviorsCodingMap)
        self.pbRemoveBehaviorsCodingMap = QtWidgets.QPushButton(self.tabBehavCodingMap)
        self.pbRemoveBehaviorsCodingMap.setObjectName("pbRemoveBehaviorsCodingMap")
        self.verticalLayout_4.addWidget(self.pbRemoveBehaviorsCodingMap)
        spacerItem11 = QtWidgets.QSpacerItem(20, 40, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.verticalLayout_4.addItem(spacerItem11)
        self.horizontalLayout.addLayout(self.verticalLayout_4)
        self.verticalLayout_8.addLayout(self.horizontalLayout)
        self.tabProject.addTab(self.tabBehavCodingMap, "")
        self.tab_time_converters = QtWidgets.QWidget()
        self.tab_time_converters.setObjectName("tab_time_converters")
        self.verticalLayout_18 = QtWidgets.QVBoxLayout(self.tab_time_converters)
        self.verticalLayout_18.setObjectName("verticalLayout_18")
        self.label_11 = QtWidgets.QLabel(self.tab_time_converters)
        self.label_11.setObjectName("label_11")
        self.verticalLayout_18.addWidget(self.label_11)
        self.horizontalLayout_16 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_16.setObjectName("horizontalLayout_16")
        self.tw_converters = QtWidgets.QTableWidget(self.tab_time_converters)
        self.tw_converters.setEditTriggers(QtWidgets.QAbstractItemView.NoEditTriggers)
        self.tw_converters.setSelectionMode(QtWidgets.QAbstractItemView.SingleSelection)
        self.tw_converters.setSelectionBehavior(QtWidgets.QAbstractItemView.SelectRows)
        self.tw_converters.setObjectName("tw_converters")
        self.tw_converters.setColumnCount(3)
        self.tw_converters.setRowCount(0)
        item = QtWidgets.QTableWidgetItem()
        self.tw_converters.setHorizontalHeaderItem(0, item)
        item = QtWidgets.QTableWidgetItem()
        self.tw_converters.setHorizontalHeaderItem(1, item)
        item = QtWidgets.QTableWidgetItem()
        self.tw_converters.setHorizontalHeaderItem(2, item)
        self.horizontalLayout_16.addWidget(self.tw_converters)
        self.verticalLayout_17 = QtWidgets.QVBoxLayout()
        self.verticalLayout_17.setObjectName("verticalLayout_17")
        self.pb_add_converter = QtWidgets.QPushButton(self.tab_time_converters)
        self.pb_add_converter.setObjectName("pb_add_converter")
        self.verticalLayout_17.addWidget(self.pb_add_converter)
        self.pb_modify_converter = QtWidgets.QPushButton(self.tab_time_converters)
        self.pb_modify_converter.setObjectName("pb_modify_converter")
        self.verticalLayout_17.addWidget(self.pb_modify_converter)
        self.pb_delete_converter = QtWidgets.QPushButton(self.tab_time_converters)
        self.pb_delete_converter.setObjectName("pb_delete_converter")
        self.verticalLayout_17.addWidget(self.pb_delete_converter)
        self.pb_load_from_file = QtWidgets.QPushButton(self.tab_time_converters)
        self.pb_load_from_file.setObjectName("pb_load_from_file")
        self.verticalLayout_17.addWidget(self.pb_load_from_file)
        self.pb_load_from_repo = QtWidgets.QPushButton(self.tab_time_converters)
        self.pb_load_from_repo.setObjectName("pb_load_from_repo")
        self.verticalLayout_17.addWidget(self.pb_load_from_repo)
        spacerItem12 = QtWidgets.QSpacerItem(20, 40, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.verticalLayout_17.addItem(spacerItem12)
        self.horizontalLayout_16.addLayout(self.verticalLayout_17)
        self.verticalLayout_18.addLayout(self.horizontalLayout_16)
        self.horizontalLayout_17 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_17.setObjectName("horizontalLayout_17")
        self.label_13 = QtWidgets.QLabel(self.tab_time_converters)
        self.label_13.setMinimumSize(QtCore.QSize(120, 0))
        self.label_13.setObjectName("label_13")
        self.horizontalLayout_17.addWidget(self.label_13)
        self.le_converter_name = QtWidgets.QLineEdit(self.tab_time_converters)
        self.le_converter_name.setObjectName("le_converter_name")
        self.horizontalLayout_17.addWidget(self.le_converter_name)
        spacerItem13 = QtWidgets.QSpacerItem(10, 20, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_17.addItem(spacerItem13)
        self.verticalLayout_18.addLayout(self.horizontalLayout_17)
        self.horizontalLayout_10 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_10.setObjectName("horizontalLayout_10")
        self.label_10 = QtWidgets.QLabel(self.tab_time_converters)
        self.label_10.setMinimumSize(QtCore.QSize(120, 0))
        self.label_10.setObjectName("label_10")
        self.horizontalLayout_10.addWidget(self.label_10)
        self.le_converter_description = QtWidgets.QLineEdit(self.tab_time_converters)
        self.le_converter_description.setObjectName("le_converter_description")
        self.horizontalLayout_10.addWidget(self.le_converter_description)
        spacerItem14 = QtWidgets.QSpacerItem(10, 20, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_10.addItem(spacerItem14)
        self.verticalLayout_18.addLayout(self.horizontalLayout_10)
        self.horizontalLayout_2 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_2.setObjectName("horizontalLayout_2")
        self.verticalLayout_9 = QtWidgets.QVBoxLayout()
        self.verticalLayout_9.setObjectName("verticalLayout_9")
        self.label_12 = QtWidgets.QLabel(self.tab_time_converters)
        self.label_12.setObjectName("label_12")
        self.verticalLayout_9.addWidget(self.label_12)
        self.pb_code_help = QtWidgets.QPushButton(self.tab_time_converters)
        self.pb_code_help.setObjectName("pb_code_help")
        self.verticalLayout_9.addWidget(self.pb_code_help)
        spacerItem15 = QtWidgets.QSpacerItem(20, 40, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.verticalLayout_9.addItem(spacerItem15)
        self.horizontalLayout_2.addLayout(self.verticalLayout_9)
        self.pteCode = QtWidgets.QPlainTextEdit(self.tab_time_converters)
        font = QtGui.QFont()
        font.setFamily("Monospace")
        self.pteCode.setFont(font)
        self.pteCode.setObjectName("pteCode")
        self.horizontalLayout_2.addWidget(self.pteCode)
        self.verticalLayout_13 = QtWidgets.QVBoxLayout()
        self.verticalLayout_13.setObjectName("verticalLayout_13")
        self.pb_save_converter = QtWidgets.QPushButton(self.tab_time_converters)
        self.pb_save_converter.setObjectName("pb_save_converter")
        self.verticalLayout_13.addWidget(self.pb_save_converter)
        self.pb_cancel_converter = QtWidgets.QPushButton(self.tab_time_converters)
        self.pb_cancel_converter.setObjectName("pb_cancel_converter")
        self.verticalLayout_13.addWidget(self.pb_cancel_converter)
        spacerItem16 = QtWidgets.QSpacerItem(20, 40, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.verticalLayout_13.addItem(spacerItem16)
        self.horizontalLayout_2.addLayout(self.verticalLayout_13)
        self.verticalLayout_18.addLayout(self.horizontalLayout_2)
        self.tabProject.addTab(self.tab_time_converters, "")
        self.verticalLayout_6.addWidget(self.tabProject)
        self.horizontalLayout_4 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_4.setObjectName("horizontalLayout_4")
        spacerItem17 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_4.addItem(spacerItem17)
        self.pbCancel = QtWidgets.QPushButton(dlgProject)
        self.pbCancel.setObjectName("pbCancel")
        self.horizontalLayout_4.addWidget(self.pbCancel)
        self.pbOK = QtWidgets.QPushButton(dlgProject)
        self.pbOK.setObjectName("pbOK")
        self.horizontalLayout_4.addWidget(self.pbOK)
        self.verticalLayout_6.addLayout(self.horizontalLayout_4)
        self.verticalLayout_7.addLayout(self.verticalLayout_6)

        self.retranslateUi(dlgProject)
        self.tabProject.setCurrentIndex(3)
        QtCore.QMetaObject.connectSlotsByName(dlgProject)

    def retranslateUi(self, dlgProject):
        _translate = QtCore.QCoreApplication.translate
        dlgProject.setWindowTitle(_translate("dlgProject", "Project"))
        self.label.setText(_translate("dlgProject", "Project name"))
        self.lbProjectFilePath.setText(_translate("dlgProject", "Project file path:"))
        self.label_7.setText(_translate("dlgProject", "Project date and time"))
        self.dteDate.setDisplayFormat(_translate("dlgProject", "yyyy-MM-dd hh:mm"))
        self.label_6.setText(_translate("dlgProject", "Project description"))
        self.lbTimeFormat.setText(_translate("dlgProject", "Project time format"))
        self.rbSeconds.setText(_translate("dlgProject", "seconds"))
        self.rbHMS.setText(_translate("dlgProject", "hh:mm:ss.mss"))
        self.lb_project_format_version.setText(_translate("dlgProject", "Project format version:"))
        self.tabProject.setTabText(self.tabProject.indexOf(self.tabInformation), _translate("dlgProject", "Information"))
        self.twBehaviors.setSortingEnabled(False)
        item = self.twBehaviors.horizontalHeaderItem(0)
        item.setText(_translate("dlgProject", "Behavior type"))
        item = self.twBehaviors.horizontalHeaderItem(1)
        item.setText(_translate("dlgProject", "Key"))
        item = self.twBehaviors.horizontalHeaderItem(2)
        item.setText(_translate("dlgProject", "Code"))
        item = self.twBehaviors.horizontalHeaderItem(3)
        item.setText(_translate("dlgProject", "Description"))
        item = self.twBehaviors.horizontalHeaderItem(4)
        item.setText(_translate("dlgProject", "Color"))
        item = self.twBehaviors.horizontalHeaderItem(5)
        item.setText(_translate("dlgProject", "Category"))
        item = self.twBehaviors.horizontalHeaderItem(6)
        item.setText(_translate("dlgProject", "Modifiers"))
        item = self.twBehaviors.horizontalHeaderItem(7)
        item.setText(_translate("dlgProject", "Exclusion"))
        item = self.twBehaviors.horizontalHeaderItem(8)
        item.setText(_translate("dlgProject", "Modifiers coding map"))
        self.pb_behavior.setText(_translate("dlgProject", "Behavior"))
        self.pb_import.setText(_translate("dlgProject", "Import ethogram"))
        self.pbBehaviorsCategories.setText(_translate("dlgProject", "Behavioral categories"))
        self.pb_exclusion_matrix.setText(_translate("dlgProject", "Exclusion matrix"))
        self.pbExportEthogram.setText(_translate("dlgProject", "Export ethogram"))
        self.lbObservationsState.setText(_translate("dlgProject", "TextLabel"))
        self.tabProject.setTabText(self.tabProject.indexOf(self.tabEthogram), _translate("dlgProject", "Ethogram"))
        self.twSubjects.setSortingEnabled(False)
        item = self.twSubjects.horizontalHeaderItem(0)
        item.setText(_translate("dlgProject", "Key"))
        item = self.twSubjects.horizontalHeaderItem(1)
        item.setText(_translate("dlgProject", "Subject name"))
        item = self.twSubjects.horizontalHeaderItem(2)
        item.setText(_translate("dlgProject", "Description"))
        self.pb_subjects.setText(_translate("dlgProject", "Subjects"))
        self.pbImportSubjectsFromProject.setText(_translate("dlgProject", "Import subjects"))
        self.pb_export_subjects.setText(_translate("dlgProject", "Export subjects"))
        self.lbSubjectsState.setText(_translate("dlgProject", "TextLabel"))
        self.tabProject.setTabText(self.tabProject.indexOf(self.tabSubjects), _translate("dlgProject", "Subjects"))
        self.twVariables.setSortingEnabled(False)
        item = self.twVariables.horizontalHeaderItem(0)
        item.setText(_translate("dlgProject", "Label"))
        item = self.twVariables.horizontalHeaderItem(1)
        item.setText(_translate("dlgProject", "Description"))
        item = self.twVariables.horizontalHeaderItem(2)
        item.setText(_translate("dlgProject", "Type"))
        item = self.twVariables.horizontalHeaderItem(3)
        item.setText(_translate("dlgProject", "Predefined value"))
        item = self.twVariables.horizontalHeaderItem(4)
        item.setText(_translate("dlgProject", "Set of values"))
        self.label_2.setText(_translate("dlgProject", "Label"))
        self.label_3.setText(_translate("dlgProject", "Description"))
        self.label_8.setText(_translate("dlgProject", "Type"))
        self.label_4.setText(_translate("dlgProject", "Predefined value"))
        self.label_9.setText(_translate("dlgProject", "Predefined timestamp"))
        self.dte_default_date.setDisplayFormat(_translate("dlgProject", "yyyy-MM-dd hh:mm:ss.zzz"))
        self.label_5.setText(_translate("dlgProject", "Set of values (separated by comma)"))
        self.pbAddVariable.setText(_translate("dlgProject", "Add variable"))
        self.pbRemoveVariable.setText(_translate("dlgProject", "Remove variable"))
        self.pbImportVarFromProject.setText(_translate("dlgProject", "Import variables\n"
"from a BORIS project"))
        self.tabProject.setTabText(self.tabProject.indexOf(self.tabIndependentVariables), _translate("dlgProject", "Independent variables"))
        self.twBehavCodingMap.setSortingEnabled(False)
        item = self.twBehavCodingMap.horizontalHeaderItem(0)
        item.setText(_translate("dlgProject", "Name"))
        item = self.twBehavCodingMap.horizontalHeaderItem(1)
        item.setText(_translate("dlgProject", "Behavior codes"))
        self.pbAddBehaviorsCodingMap.setText(_translate("dlgProject", "Add a behaviors coding map"))
        self.pbRemoveBehaviorsCodingMap.setText(_translate("dlgProject", "Remove behaviors coding map"))
        self.tabProject.setTabText(self.tabProject.indexOf(self.tabBehavCodingMap), _translate("dlgProject", "Behaviors coding map"))
        self.label_11.setText(_translate("dlgProject", "Time converters for external data"))
        self.tw_converters.setSortingEnabled(False)
        item = self.tw_converters.horizontalHeaderItem(0)
        item.setText(_translate("dlgProject", "Name"))
        item = self.tw_converters.horizontalHeaderItem(1)
        item.setText(_translate("dlgProject", "Description"))
        item = self.tw_converters.horizontalHeaderItem(2)
        item.setText(_translate("dlgProject", "Code"))
        self.pb_add_converter.setText(_translate("dlgProject", "Add new converter"))
        self.pb_modify_converter.setText(_translate("dlgProject", "Modify converter"))
        self.pb_delete_converter.setText(_translate("dlgProject", "Delete converter"))
        self.pb_load_from_file.setText(_translate("dlgProject", "Load converters from file"))
        self.pb_load_from_repo.setText(_translate("dlgProject", "Load converters from BORIS repository"))
        self.label_13.setText(_translate("dlgProject", "Name"))
        self.label_10.setText(_translate("dlgProject", "Description"))
        self.label_12.setText(_translate("dlgProject", "Python code"))
        self.pb_code_help.setText(_translate("dlgProject", "Help"))
        self.pb_save_converter.setText(_translate("dlgProject", "Save converter"))
        self.pb_cancel_converter.setText(_translate("dlgProject", "Cancel"))
        self.tabProject.setTabText(self.tabProject.indexOf(self.tab_time_converters), _translate("dlgProject", "Converters"))
        self.pbCancel.setText(_translate("dlgProject", "Cancel"))
        self.pbOK.setText(_translate("dlgProject", "OK"))



"""
BORIS
Behavioral Observation Research Interactive Software
Copyright 2012-2024 Olivier Friard

This file is part of BORIS.

  BORIS is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 3 of the License, or
  any later version.

  BORIS is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not see <http://www.gnu.org/licenses/>.

"""

import logging
from PyQt5.QtWidgets import (
    QDialog,
    QVBoxLayout,
    QHBoxLayout,
    QPushButton,
    QLabel,
    QTableWidget,
    QSpacerItem,
    QAbstractItemView,
    QSizePolicy,
)


class ExclusionMatrix(QDialog):
    def __init__(self):
        super().__init__()

        hbox = QVBoxLayout(self)

        self.label = QLabel()
        self.label.setText(
            ("Check if behaviors are mutually exclusive.\n" "The Point events (displayed on blue background) cannot be excluded)")
        )
        hbox.addWidget(self.label)

        self.twExclusions = QTableWidget()
        self.twExclusions.setSelectionBehavior(QAbstractItemView.SelectRows)
        self.twExclusions.setAlternatingRowColors(True)
        self.twExclusions.setEditTriggers(QAbstractItemView.NoEditTriggers)
        hbox.addWidget(self.twExclusions)

        hbox2 = QHBoxLayout()
        spacer_item = QSpacerItem(40, 20, QSizePolicy.Expanding, QSizePolicy.Minimum)
        hbox2.addItem(spacer_item)

        self.pb_select_all = QPushButton("Check all")
        self.pb_select_all.clicked.connect(lambda: self.pb_cb_selection("select"))
        hbox2.addWidget(self.pb_select_all)

        self.pb_unselect_all = QPushButton("Uncheck all")
        self.pb_unselect_all.clicked.connect(lambda: self.pb_cb_selection("unselect"))
        hbox2.addWidget(self.pb_unselect_all)

        self.pb_revert_selection = QPushButton("Revert check")
        self.pb_revert_selection.clicked.connect(lambda: self.pb_cb_selection("revert"))
        hbox2.addWidget(self.pb_revert_selection)

        self.pb_check_selected = QPushButton("Check selected")
        self.pb_check_selected.clicked.connect(lambda: self.pb_selected(True))
        hbox2.addWidget(self.pb_check_selected)

        self.pb_uncheck_selected = QPushButton("Uncheck selected")
        self.pb_uncheck_selected.clicked.connect(lambda: self.pb_selected(False))
        hbox2.addWidget(self.pb_uncheck_selected)

        self.pbCancel = QPushButton("Cancel")
        self.pbCancel.clicked.connect(self.reject)
        hbox2.addWidget(self.pbCancel)

        self.pbOK = QPushButton("OK", clicked=self.accept)
        hbox2.addWidget(self.pbOK)

        hbox.addLayout(hbox2)
        self.setLayout(hbox)

        self.setWindowTitle("Behaviors exclusion matrix")
        self.setGeometry(100, 100, 600, 400)

    def pb_selected(self, to_check: bool):
        """
        check/uncheck the checkbox in selected cells

        Args:
            to_check (boolean): True to check else False
        """
        for selected_range in self.twExclusions.selectedRanges():
            for row in range(selected_range.topRow(), selected_range.bottomRow() + 1):
                for column in range(self.twExclusions.columnCount()):
                    try:
                        self.twExclusions.cellWidget(row, column).setChecked(to_check)
                    except Exception:
                        logging.warning(f"Error during checking/unchecking for {row}/{column} in exclusion matrix")
        self.cb_clicked()

    def pb_cb_selection(self, mode):
        """
        button for checkbox selection/deselection and revert selection

        Args:
            mode (str): select, unselect, revert

        """

        for r in range(self.twExclusions.rowCount()):
            for c in range(self.twExclusions.columnCount()):
                if mode == "select":
                    state = True
                if mode == "unselect":
                    state = False
                try:
                    if mode == "revert":
                        state = not self.twExclusions.cellWidget(r, c).isChecked()
                    self.twExclusions.cellWidget(r, c).setChecked(state)
                except Exception:
                    logging.warning(f"Error during checking/unchecking for {r}/{c} in exclusion matrix")

    def cb_clicked(self):
        """
        de/select the corresponding checkbox
        """
        for r, r_name in enumerate(self.stateBehaviors + self.point_behaviors):
            for c, c_name in enumerate(self.stateBehaviors):
                if c_name != r_name:
                    try:
                        if f"{c_name}|{r_name}" in self.checkboxes:
                            self.checkboxes[f"{c_name}|{r_name}"].setChecked(self.checkboxes[f"{r_name}|{c_name}"].isChecked())
                    except Exception:
                        logging.warning(f"Error during checking/unchecking for {r_name}/{c_name} in exclusion matrix")



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""Utilities for compiling SASS files."""

# Standard library imports
import keyword
import logging
import os
import re
import shutil
import sys

# Third party imports
import qtsass

# Local imports
from qdarkstyle import (MAIN_SCSS_FILE, PACKAGE_PATH, QSS_FILE_SUFFIX, QSS_PATH,
                        VARIABLES_SCSS_FILE)
from qdarkstyle.palette import Palette
from qdarkstyle.utils.images import create_images, create_palette_image

# Constants
HEADER_SCSS = '''// ---------------------------------------------------------------------------
//
//    WARNING! File created programmatically. All changes made in this file will be lost!
//
//    Created by the qtsass compiler v{}
//
//    The definitions are in the "qdarkstyle.palette" module
//
//----------------------------------------------------------------------------
'''

HEADER_QSS = '''/* ---------------------------------------------------------------------------

    WARNING! File created programmatically. All changes made in this file will be lost!

    Created by the qtsass compiler v{}

    The definitions are in the "qdarkstyle.qss._styles.scss" module

--------------------------------------------------------------------------- */
'''

_logger = logging.getLogger(__name__)


def _dict_to_scss(data):
    """Create a scss variables string from a dict."""
    lines = []
    template = "${}: {};"
    for key, value in data.items():
        line = template.format(key, value)
        lines.append(line)

    return '\n'.join(lines)


def _scss_to_dict(string):
    """Parse variables and return a dict."""
    data = {}
    lines = string.split('\n')

    for line in lines:
        line = line.strip()

        if line and line.startswith('$'):
            key, value = line.split(':')
            key = key[1:].strip()
            key = key.replace('-', '_')
            value = value.split(';')[0].strip()

            data[key] = value

    return data


def _create_scss_variables(variables_scss_filepath, palette,
                           header=HEADER_SCSS):
    """Create a scss variables file."""

    scss = _dict_to_scss(palette.to_dict())
    data = header.format(qtsass.__version__) + scss + '\n'

    _logger.info("Generating SCSS variables file ...")
    _logger.info(f"File path: {variables_scss_filepath}")

    with open(variables_scss_filepath, 'w') as f:
        f.write(data)


def _create_qss(main_scss_path, qss_filepath, header=HEADER_QSS):
    """Create a styles.qss file from qtsass."""

    data = ''

    _logger.info("Generating QSS file ...")
    _logger.info(f"SCSS path: {main_scss_path}")
    _logger.info(f"QSS path: {qss_filepath}")

    qtsass.compile_filename(main_scss_path, qss_filepath,
                            output_style='expanded')

    with open(qss_filepath, 'r') as f:
        data = f.read()

    data = header.format(qtsass.__version__) + data

    with open(qss_filepath, 'w') as f:
        f.write(data)

    return data


def create_qss(palette, base_path=''):
    """Create variables files and run qtsass compilation.

    This function will use the structure that must contain::

        base_path
            [palette.ID]
                main.scss
            qss
                _style.scss

    The structure after the execution will contain::

        base_path
            [palette.ID]
                _variables.scss
                [palette.ID]style.qss
                main.scss
            qss
                _style.scss

    Args:
        palette (Palette): Palette class.
        base_path (str): Base path for the palette directory, required for
            custom palettes. Defaults to '' that uses `[PACKAGE_PATH]`.

    Returns:
        str: Stylesheet in string format.
    """

    if palette.ID is None:
        print("A QDarkStyle palette requires an ID!")
        sys.exit(1)

    palette_path = os.path.join(PACKAGE_PATH, palette.ID)

    if base_path:
        palette_path = os.path.join(base_path, palette.ID)

    _logger.info(f"Creating QSS for palette: '{palette.ID} ...")
    _logger.info(f"Palette path: {palette_path}")

    variables_scss_filepath = os.path.join(palette_path, VARIABLES_SCSS_FILE)
    main_scss_filepath = os.path.join(palette_path, MAIN_SCSS_FILE)
    qss_filepath = os.path.join(palette_path, palette.ID + QSS_FILE_SUFFIX)

    _create_scss_variables(variables_scss_filepath, palette)

    stylesheet = _create_qss(main_scss_filepath, qss_filepath)

    return stylesheet


def create_custom_qss(
    id,
    path,
    color_backgroung=[],  # 6 elements
    color_text=[],  # 4 elements
    color_accent=[],  # 4 elemetns
    border=[],  # 3 elements
    border_selection=[],  # 3 elements
    opacity_tooltip=None,
    size_border_radius=None,
    w_status_bar_background_color=None,
    path_resources=None
):
    """
    Create a custom palette based on the parameters defined.

    The `id` must be a valid Python identifier and will be stored
    as a lowercased folder (even if the identifier had uppercase letters).

    This function returns the custom stylesheet pointing to resources stored
    at path_resources/[id].
    """

    stylesheet = ''

    # Check if name/id is valid
    if id.isidentifier():
        name = id if id[0].isupper() else id.capitalize()
    else:
        raise Exception('The custom palette name must be a valid Python '
                        'identifier!')

    # Copy resources folder
    rc_loc = os.path.basename(RC_PATH)
    qss_loc = os.path.basename(QSS_PATH)
    theme_root_path = os.path.join(path, name.lower())
    theme_rc_path = os.path.join(theme_root_path, rc_loc)

    if os.path.isdir(theme_root_path):
        shutil.rmtree(theme_root_path)

    shutil.copytree(RC_PATH, theme_rc_path)

    # Copy QSS folder and contents
    theme_qss_path = os.path.join(theme_root_path, qss_loc)

    if os.path.isdir(theme_qss_path):
        os.removedirs(theme_qss_path)

    shutil.copytree(QSS_PATH, theme_qss_path)

    # Create custom palette
    # ATTENTION: here we need to iterate over
    # args of this function to fill the values
    # of the Palette class member with correct values
    # they still using the old values below
    custom_palette = type(name, (Palette, ), {})
    custom_palette.COLOR_BACKGROUND_LIGHT = color_background_light
    custom_palette.COLOR_BACKGROUND_NORMAL = color_background_normal
    custom_palette.COLOR_BACKGROUND_DARK = color_background_dark
    custom_palette.COLOR_FOREGROUND_LIGHT = color_foreground_light
    custom_palette.COLOR_FOREGROUND_NORMAL = color_foreground_normal
    custom_palette.COLOR_FOREGROUND_DARK = color_foreground_dark
    custom_palette.COLOR_SELECTION_LIGHT = color_selection_light
    custom_palette.COLOR_SELECTION_NORMAL = color_selection_normal
    custom_palette.COLOR_SELECTION_DARK = color_selection_dark
    custom_palette.SIZE_BORDER_RADIUS = border_radius
    custom_palette.PATH_RESOURCES = "'{}'".format(theme_root_path)

    # Process images and save them to the custom platte rc folder
    create_images(rc_path=theme_rc_path, palette=custom_palette)
    create_palette_image(path=theme_root_path, palette=custom_palette)

    # Compile SCSS
    variables_scss_filepath = os.path.join(theme_qss_path, VARIABLES_SCSS_FILE)
    theme_main_scss_filepath = os.path.join(theme_qss_path, MAIN_SCSS_FILE)
    theme_qss_filepath = os.path.join(theme_root_path, palette.ID + QSS_FILE_SUFFIX)

    stylesheet = create_qss(
        qss_filepath=theme_qss_filepath,
        main_scss_filepath=theme_main_scss_filepath,
        variables_scss_filepath=variables_scss_filepath,
        palette=custom_palette,
    )

    # Update colors in text
    with open(theme_main_scss_filepath, 'r') as fh:
        data = fh.read()

    for key, color in Palette.color_palette().items():
        custom_color = custom_palette.color_palette()[key].upper()
        data = data.replace(color, custom_color)
        stylesheet = stylesheet.replace(color, custom_color)

    with open(theme_main_scss_filepath, 'w') as fh:
        fh.write(data)

    with open(theme_qss_filepath, 'w') as fh:
        fh.write(stylesheet)

    return stylesheet


def create_custom_qss_from_palette(name, path, palette):
    """
    Create a custom palette based on a palette class.
    """
    kwargs = {
        'name': name,
        'path': path,
        'border_radius': palette.SIZE_BORDER_RADIUS,
    }
    kwargs.update(palette.color_palette())
    stylesheet = create_custom_qss(**kwargs)

    return stylesheet


def create_custom_qss_from_dict(name, path, palette_dict):
    """
    Create a custom palette based on a palette dictionary.
    """
    kwargs = {
        'name': name,
        'path': path,
        'border_radius': palette_dict.get('SIZE_BORDER_RADIUS', '4px'),
    }
    kwargs.update(palette_dict)
    stylesheet = create_custom_qss(**kwargs)

    return stylesheet


if __name__ == '__main__':
    # Example of a custom palette
    # TODO: change to not use a specific path
    # TODO: may move to other place, e.g., example.py
    qss = create_custom_qss(
        'MyAwesomePalette',
        '/Users/gpena-castellanos/Desktop',
        '#ff0000',
        '#cc0000',
        '#aa0000',
        '#00ff00',
        '#00cc00',
        '#00aa00',
        '#0000ff',
        '#0000cc',
        '#0000aa',
        '0px',
    )



# -*- coding: utf-8 -*-
"""Script to process QRC files (convert .qrc to _rc.py and .rcc).

The script will attempt to compile the qrc file using the following tools:

    - `pyside6-rcc` for PySide6 and QtPy (Python) (Official)
    - There is no specific rcc compiler for PyQt6, use `pyside6-rcc` (Python)
    - `pyrcc5` for PyQt5 (Python)
    - `pyside2-rcc` for PySide2 (Python)
    - `rcc` for Qt5/Qt6 (C++)

Delete the compiled files that you don't want to use manually after
running this script.

Links to understand those tools:

    - `pyside6-rcc`: https://doc.qt.io/qtforpython/tutorials/basictutorial/qrcfiles.html (Official)
    - `pyrcc5`: http://pyqt.sourceforge.net/Docs/PyQt5/resources.html#pyrcc5
    - `pyside2-rcc: https://doc.qt.io/qtforpython/overviews/resources.html (Documentation Incomplete)
    - `rcc` on Qt6: https://doc.qt.io/qt-6/resources.html
    - `rcc` on Qt5: http://doc.qt.io/qt-5/rcc.html

"""

# Standard library imports
import argparse
import logging
import sys

# Third party imports
from watchdog.events import FileSystemEventHandler
from watchdog.observers import Observer

# Local imports
from qdarkstyle import PACKAGE_PATH
from qdarkstyle.dark.palette import DarkPalette
from qdarkstyle.light.palette import LightPalette
from qdarkstyle.utils import process_palette

_logger = logging.getLogger(__name__)


class QSSFileHandler(FileSystemEventHandler):
    """QSS File observer."""

    def __init__(self, parser_args):
        """QSS File observer."""
        super(QSSFileHandler, self).__init__()
        self.args = parser_args

    def on_modified(self, event):
        """Handle file system events."""
        if event.src_path.endswith('.qss'):
            # TODO: needs implementation for new palettes
            process_palette(compile_for=self.args.create)
            print('\n')


def main():
    """Process QRC files."""
    parser = argparse.ArgumentParser(description=__doc__,
                                     formatter_class=argparse.RawDescriptionHelpFormatter)
    parser.add_argument('--qrc_dir',
                        default=None,
                        type=str,
                        help="QRC file directory, relative to current directory.",)
    parser.add_argument('--create',
                        default='qtpy',
                        choices=['pyqt5', 'pyqt6', 'pyside2', 'pyside6', 'qtpy', 'pyqtgraph', 'qt', 'qt5', 'all'],
                        type=str,
                        help="Choose which one would be generated.")
    parser.add_argument('--watch', '-w',
                        action='store_true',
                        help="Watch for file changes.")

    args = parser.parse_args()

    if args.watch:
        path = PACKAGE_PATH
        observer = Observer()
        handler = QSSFileHandler(parser_args=args)
        observer.schedule(handler, path, recursive=True)
        try:
            print('\nWatching QSS file for changes...\nPress Ctrl+C to exit\n')
            observer.start()
        except KeyboardInterrupt:
            observer.stop()
        observer.join()
    else:
        for palette in [DarkPalette, LightPalette]:
            process_palette(palette=palette, compile_for=args.create)


if __name__ == "__main__":
    logging.basicConfig(level=logging.DEBUG)
    sys.exit(main())



"""
BORIS
Behavioral Observation Research Interactive Software
Copyright 2012-2024 Olivier Friard

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
  MA 02110-1301, USA.


Module for analyzing the co-occurence of behaviors

"""

from . import config as cfg
from . import select_subj_behav
from . import dialog
from . import utilities as util
from . import select_observations

from . import project_functions, observation_operations

from PyQt5.QtWidgets import QMessageBox
from PyQt5.QtGui import QFont, QTextOption
from . import portion as I
import itertools
import logging
from decimal import Decimal as dec


def get_cooccurence(self):
    """
    get co-occurence of selected behaviors
    """

    QMessageBox.warning(
        None,
        cfg.programName,
        (
            "This function is experimental. Please test it and report any bug and suggestions at <br>"
            '<a href="https://github.com/olivierfriard/BORIS/issues">'
            "https://github.com/olivierfriard/BORIS/issues</a><br>"
            "Thank you for your collaboration!"
        ),
        QMessageBox.Ok | QMessageBox.Default,
        QMessageBox.NoButton,
    )

    def interval_len(interval: I) -> dec:
        """ "
        returns duration of an interval or a set of intervals
        """
        if interval.empty:
            return dec(0)
        else:
            return dec(sum([x.upper - x.lower for x in interval]))

    _, selected_observations = select_observations.select_observations2(
        self, cfg.MULTIPLE, windows_title="Select the observations for behaviors co-occurence analysis"
    )

    if not selected_observations:
        return

    # check if coded behaviors are defined in ethogram
    if project_functions.check_coded_behaviors_in_obs_list(self.pj, selected_observations):
        return

    # check if state events are paired
    not_ok, selected_observations = project_functions.check_state_events(self.pj, selected_observations)
    if not_ok or not selected_observations:
        return

    start_coding, end_coding, _ = observation_operations.coding_time(self.pj[cfg.OBSERVATIONS], selected_observations)
    # exit with message if events do not have timestamp
    if start_coding.is_nan():
        QMessageBox.critical(
            None,
            cfg.programName,
            ("This function is not available for observations with events that do not have timestamp"),
            QMessageBox.Ok | QMessageBox.Default,
            QMessageBox.NoButton,
        )
        return

    while True:
        flag_ok: bool = True
        parameters = select_subj_behav.choose_obs_subj_behav_category(
            self,
            selected_observations,
            window_title="Select the behaviors",
            n_observations=len(selected_observations),
            flagShowIncludeModifiers=False,
            flagShowExcludeBehaviorsWoEvents=True,
        )

        if parameters == {}:  # cancel button pressed
            return

        if not parameters[cfg.SELECTED_SUBJECTS]:
            QMessageBox.warning(None, cfg.programName, "Select the subject(s) to analyze")
            flag_ok = False

        # check number of behaviors (must be <=4)
        if flag_ok and len(parameters[cfg.SELECTED_BEHAVIORS]) > 4:
            QMessageBox.warning(None, cfg.programName, "You cannot select more than 4 behaviors")
            flag_ok = False

        # check number of behaviors (must be > 1)
        if flag_ok and len(parameters[cfg.SELECTED_BEHAVIORS]) < 2:
            QMessageBox.warning(None, cfg.programName, "You must select almost 2 behaviors")
            flag_ok = False

        if flag_ok:
            break

    logging.debug(f"{parameters[cfg.SELECTED_BEHAVIORS]}")

    state_events_list = util.state_behavior_codes(self.pj[cfg.ETHOGRAM])

    events_interval: dict = {}
    mem_events_interval: dict = {}

    for obs_id in selected_observations:
        events_interval[obs_id] = {}
        mem_events_interval[obs_id] = {}

        for event in self.pj[cfg.OBSERVATIONS][obs_id][cfg.EVENTS]:
            if event[cfg.EVENT_SUBJECT_FIELD_IDX] not in events_interval[obs_id]:
                events_interval[obs_id][event[cfg.EVENT_SUBJECT_FIELD_IDX]] = {}
                mem_events_interval[obs_id][event[cfg.EVENT_SUBJECT_FIELD_IDX]] = {}

            if event[cfg.EVENT_BEHAVIOR_FIELD_IDX] not in events_interval[obs_id][event[cfg.EVENT_SUBJECT_FIELD_IDX]]:
                events_interval[obs_id][event[cfg.EVENT_SUBJECT_FIELD_IDX]][event[cfg.EVENT_BEHAVIOR_FIELD_IDX]] = I.empty()
                mem_events_interval[obs_id][event[cfg.EVENT_SUBJECT_FIELD_IDX]][event[cfg.EVENT_BEHAVIOR_FIELD_IDX]] = []

            # state event
            if event[cfg.EVENT_BEHAVIOR_FIELD_IDX] in state_events_list:
                mem_events_interval[obs_id][event[cfg.EVENT_SUBJECT_FIELD_IDX]][event[cfg.EVENT_BEHAVIOR_FIELD_IDX]].append(
                    event[cfg.EVENT_TIME_FIELD_IDX]
                )
                if len(mem_events_interval[obs_id][event[cfg.EVENT_SUBJECT_FIELD_IDX]][event[cfg.EVENT_BEHAVIOR_FIELD_IDX]]) == 2:
                    events_interval[obs_id][event[cfg.EVENT_SUBJECT_FIELD_IDX]][event[cfg.EVENT_BEHAVIOR_FIELD_IDX]] |= I.closedopen(
                        mem_events_interval[obs_id][event[cfg.EVENT_SUBJECT_FIELD_IDX]][event[cfg.EVENT_BEHAVIOR_FIELD_IDX]][0],
                        mem_events_interval[obs_id][event[cfg.EVENT_SUBJECT_FIELD_IDX]][event[cfg.EVENT_BEHAVIOR_FIELD_IDX]][1],
                    )
                    mem_events_interval[obs_id][event[cfg.EVENT_SUBJECT_FIELD_IDX]][event[cfg.EVENT_BEHAVIOR_FIELD_IDX]] = []
            # point event
            else:
                events_interval[obs_id][event[cfg.EVENT_SUBJECT_FIELD_IDX]][event[cfg.EVENT_BEHAVIOR_FIELD_IDX]] |= I.singleton(
                    event[cfg.EVENT_TIME_FIELD_IDX]
                )

    logging.debug(f"events_interval: {events_interval}")

    cooccurence_results: dict = {}

    for obs_id in selected_observations:
        logging.debug(f"obs_id: {obs_id}")

        for subject in parameters[cfg.SELECTED_SUBJECTS]:
            if subject == "No focal subject":
                subj = ""
            else:
                subj = subject

            if subject not in cooccurence_results:
                cooccurence_results[subject] = {}

            logging.debug(f"subject {subject}")

            for n_combinations in range(2, len(parameters[cfg.SELECTED_BEHAVIORS]) + 1):
                union = I.empty()

                logging.debug(f"{n_combinations=}")

                for combination in itertools.combinations(parameters[cfg.SELECTED_BEHAVIORS], n_combinations):
                    logging.debug(f"{combination=}")
                    if subj in events_interval[obs_id]:
                        # init
                        if combination[0] in events_interval[obs_id][subj]:
                            union = events_interval[obs_id][subj][combination[0]]
                        else:
                            union = I.empty()

                        logging.debug(f"{combination[0]=} {union=}")

                        for combination2 in combination[1:]:
                            if combination2 in events_interval[obs_id][subj]:
                                inter2 = events_interval[obs_id][subj][combination2]
                            else:
                                inter2 = I.empty()

                            logging.debug(f"{combination2=} {inter2=}")

                            union &= inter2

                        if combination not in cooccurence_results[subject]:
                            cooccurence_results[subject][combination] = 0

                        logging.debug(f"{combination=} {union=}")
                        cooccurence_results[subject][combination] += interval_len(union)
                    else:
                        if combination not in cooccurence_results[subject]:
                            cooccurence_results[subject][combination] = 0
                        cooccurence_results[subject][combination] += 0

                    logging.debug(f"{cooccurence_results[subject][combination]=}")

    logging.debug(cooccurence_results)

    out = f"<b>Co-occurence of behaviors: {','.join(parameters[cfg.SELECTED_BEHAVIORS])}</b><br><br>"
    for subject in parameters[cfg.SELECTED_SUBJECTS]:
        out += f"<br>Subject <b>{subject}</b><br><br>"
        for combination in cooccurence_results[subject]:
            if parameters[cfg.EXCLUDE_BEHAVIORS] and not cooccurence_results[subject][combination]:
                continue
            duration = f"<b>{cooccurence_results[subject][combination]}</b>" if cooccurence_results[subject][combination] else "0"
            out += f"<b>{'</b> and <b>'.join(combination)}</b>: {duration} s<br>"

    self.results = dialog.Results_dialog()
    self.results.setWindowTitle("Behaviors co-occurence")
    self.results.ptText.setFont(QFont("Courier", 12))
    self.results.ptText.setWordWrapMode(QTextOption.NoWrap)
    self.results.ptText.setReadOnly(True)
    self.results.ptText.clear()
    self.results.ptText.appendHtml(out)
    self.results.show()



"""
BORIS
Behavioral Observation Research Interactive Software
Copyright 2012-2022 Olivier Friard

module for testing db_functions.py

pytest -vv test_db_functions.py
"""

import os
import sys
import json

sys.path.insert(0, os.path.abspath(os.path.join(os.path.dirname(__file__), "..")))

from boris import db_functions


class Test_load_events_in_db(object):
    def test_1(self):

        pj = json.loads(open("files/test.boris").read())

        cursor = db_functions.load_events_in_db(pj, ["subject1"], ["observation #1"], ["s"])
        cursor.execute(
            "SELECT occurence FROM events WHERE observation = ? AND subject = ? AND code = ?",
            ("observation #1", "subject1", "s"),
        )
        out = ""
        for r in cursor.fetchall():
            out += "{}\n".format(r[0])

        REF = """3.3\n7.75\n9.9\n16.2\n18.35\n24.475\n"""
        assert out == REF

    def test_2(self):

        pj = json.loads(open("files/test.boris").read())

        cursor = db_functions.load_events_in_db(pj, ["subject2"], ["live"], ["s", "p"])
        cursor.execute(
            "SELECT occurence FROM events WHERE observation = ? AND subject = ? AND code = ?", ("live", "subject2", "s")
        )
        out = ""
        for r in cursor.fetchall():
            out += "{}\n".format(r[0])

        REF = """12.509\n20.685\n"""

        assert out == REF

    def test_3(self):
        """
        no focal subject, observation with not paired events
        """

        pj = json.loads(open("files/test.boris").read())

        cursor = db_functions.load_events_in_db(pj, ["No focal subject"], ["live not paired"], ["s", "p"])
        cursor.execute(
            "SELECT occurence FROM events WHERE observation = ? AND subject = ? AND code = ?",
            ("live not paired", "No focal subject", "s"),
        )
        out = ""
        for r in cursor.fetchall():
            out += "{}\n".format(r[0])

        REF = """2.718\n10.478\n12.926\n17.47\n19.502\n24.318\n26.862\n"""

        assert out == REF


class Test_load_aggregated_events_in_db(object):
    def test_dump(self):

        pj = json.loads(open("files/test.boris").read())

        ok, msg, db = db_functions.load_aggregated_events_in_db(pj, [], ["observation #1", "observation #2"], [])
        out = ""
        for line in db.iterdump():
            out += line + "\n"

        print(out == open("files/test_db_functions_test1").read())

    def test_not_ok(self):
        """
        test with observation with state events NOT PAIRED
        """

        pj = json.loads(open("files/test.boris").read())

        ok, msg, db = db_functions.load_aggregated_events_in_db(pj, [], ["live not paired"], [])

        assert ok == False

    def test_no_observation(self):
        """
        test with no observation
        """

        pj = json.loads(open("files/test.boris").read())

        ok, msg, db = db_functions.load_aggregated_events_in_db(pj, [], [], [])

        assert ok == False
